@@ -18,13 +18,11 @@ class GenModel(base.Generative):
 
     Parameters
     ----------
-    degree : int, optional
-        a positive integer. Default is None, in which case 
-        a value consistent with ``theta_vec``, ``h_mu``, 
-        and ``h_lambda_mat`` is used. If all of them are not given,
-        degree is assumed to be 1.
+    c_degree : int
+        a positive integer.
     theta_vec : numpy ndarray, optional
-        a vector of real numbers, which includs the constant term, by default [0.0, 0.0, ... , 0.0]
+        a vector of real numbers, which includs the constant term, 
+        by default [0.0, 0.0, ... , 0.0]
     tau : float, optional
         a positive real number, by default 1.0
     h_mu_vec : numpy ndarray, optional
@@ -40,108 +38,81 @@ class GenModel(base.Generative):
         by default None
     """
     def __init__(
-        self,
-        *,
-        degree=None,
-        theta_vec=None,
-        tau=1.0,
-        h_mu_vec=None,
-        h_lambda_mat=None,
-        h_alpha=1.0,
-        h_beta=1.0,
-        seed=None
-        ):
-        if degree is not None:
-            self.degree = _check.nonneg_int(degree,'degree',ParameterFormatError)
-            if theta_vec is None:
-                self.theta_vec = np.zeros(self.degree+1)
-            else:
-                self.theta_vec = _check.float_vec(theta_vec,'theta_vec',ParameterFormatError)
-
-            if h_mu_vec is None:
-                self.h_mu_vec = np.zeros(self.degree+1)
-            else:
-                self.h_mu_vec = _check.float_vec(h_mu_vec,'h_mu_vec',ParameterFormatError)
-
-            if h_lambda_mat is None:
-                self.h_lambda_mat = np.identity(self.degree+1)
-            else:
-                self.h_lambda_mat = _check.pos_def_sym_mat(h_lambda_mat,'h_lambda_mat',ParameterFormatError)
-
-        elif theta_vec is not None:
-            self.theta_vec = _check.float_vec(theta_vec,'theta_vec',ParameterFormatError)
-            self.degree = self.theta_vec.shape[0]-1
-            if h_mu_vec is None:
-                self.h_mu_vec = np.zeros(self.degree+1)
-            else:
-                self.h_mu_vec = _check.float_vec(h_mu_vec,'h_mu_vec',ParameterFormatError)
-
-            if h_lambda_mat is None:
-                self.h_lambda_mat = np.identity(self.degree+1)
-            else:
-                self.h_lambda_mat = _check.pos_def_sym_mat(h_lambda_mat,'h_lambda_mat',ParameterFormatError)
-
-        elif h_mu_vec is not None:
-            self.h_mu_vec = _check.float_vec(h_mu_vec,'h_mu_vec',ParameterFormatError)
-            self.degree = self.h_mu_vec.shape[0]-1
-            self.theta_vec = np.zeros(self.degree+1)
-            if h_lambda_mat is None:
-                self.h_lambda_mat = np.identity(self.degree+1)
-            else:
-                self.h_lambda_mat = _check.pos_def_sym_mat(h_lambda_mat,'h_lambda_mat',ParameterFormatError)
-        
-        elif h_lambda_mat is not None:
-            self.h_lambda_mat = _check.pos_def_sym_mat(h_lambda_mat,'h_lambda_mat',ParameterFormatError)
-            self.degree = self.h_lambda_mat.shape[0]-1
-            self.theta_vec = np.zeros(self.degree+1)
-            self.h_mu_vec = np.zeros(self.degree+1)
-        
-        else:
-            self.degree = 1
-            self.theta_vec = np.zeros(self.degree+1)
-            self.h_mu_vec = np.zeros(self.degree+1)
-            self.h_lambda_mat = np.identity(self.degree+1)
-
-        if (self.degree+1 != self.theta_vec.shape[0]
-            or self.degree+1 != self.h_mu_vec.shape[0]
-            or self.degree+1 != self.h_lambda_mat.shape[0]):
-                raise(ParameterFormatError(
-                    "(degree+1) and dimensions of theta_vec, h_mu_vec,"
-                    +" and h_lambda_mat must be the same,"
-                    +" if two or more of them are specified."))
-
-        self.tau = _check.pos_float(tau,'tau',ParameterFormatError)
-        self.h_alpha = _check.pos_float(h_alpha,'h_alpha',ParameterFormatError)
-        self.h_beta = _check.pos_float(h_beta,'h_beta',ParameterFormatError)
+            self,
+            c_degree,
+            theta_vec=None,
+            tau=1.0,
+            h_mu_vec=None,
+            h_lambda_mat=None,
+            h_alpha=1.0,
+            h_beta=1.0,
+            seed=None
+            ):
+        # constants
+        self.c_degree = _check.nonneg_int(c_degree,'c_degree',ParameterFormatError)
         self.rng = np.random.default_rng(seed)
 
-    def set_h_params(self,h_mu_vec,h_lambda_mat,h_alpha,h_beta):
+        # params
+        self.theta_vec = np.zeros(self.c_degree+1)
+        self.tau = 1.0
+
+        # h_params
+        self.h_mu_vec = np.zeros(self.c_degree+1)
+        self.h_lambda_mat = np.eye(self.c_degree+1)
+        self.h_alpha = 1.0
+        self.h_beta = 1.0
+
+        self.set_params(theta_vec,tau)
+        self.set_h_params(h_mu_vec,h_lambda_mat,h_alpha,h_beta)
+
+    def get_constants(self):
+        """Get constants of GenModel.
+
+        Returns
+        -------
+        constants : dict of {str: int}
+            * ``"c_degree"`` : the value of ``self.c_degree``
+        """
+        return {'c_degree':self.c_degree}
+
+    def set_h_params(self,h_mu_vec=None,h_lambda_mat=None,h_alpha=None,h_beta=None):
         """Set the hyperparameters of the prior distribution.
         
         Parameters
         ----------
-        h_mu_vec : numpy ndarray
-            a vector of real numbers
-        h_lambda_mat : numpy ndarray
-            a positibe definate matrix
-        h_alpha : float
-            a positive real number
-        h_beta : float
-            a positibe real number
+        h_mu_vec : numpy ndarray, optional
+            a vector of real numbers, by default None.
+        h_lambda_mat : numpy ndarray, optional
+            a positive definate matrix, by default None.
+        h_alpha : float, optional
+            a positive real number, by default None.
+        h_beta : float, optional
+            a positive real number, by default None.
         """
-        self.h_mu_vec = _check.float_vec(h_mu_vec,'h_mu_vec',ParameterFormatError)
-        self.h_lambda_mat = _check.pos_def_sym_mat(h_lambda_mat,'h_lambda_mat',ParameterFormatError)
-        self.h_alpha = _check.pos_float(h_alpha,'h_alpha',ParameterFormatError)
-        self.h_beta = _check.pos_float(h_beta,'h_beta',ParameterFormatError)
+        if h_mu_vec is not None:
+            _check.float_vec(h_mu_vec,'h_mu_vec',ParameterFormatError)
+            _check.shape_consistency(
+                h_mu_vec.shape[0],'h_mu_vec.shape[0]',
+                self.c_degree+1,'self.c_degree+1',
+                ParameterFormatError
+            )
+            self.h_mu_vec[:] = h_mu_vec
 
-        if (self.h_mu_vec.shape[0] != self.h_lambda_mat.shape[0]):
-                raise(ParameterFormatError(
-                    "dimensions of h_mu_vec and h_lambda_mat must be the same."))
+        if h_lambda_mat is not None:
+            _check.pos_def_sym_mat(h_lambda_mat,'h_lambda_mat',ParameterFormatError)
+            _check.shape_consistency(
+                h_lambda_mat.shape[0],'h_lambda_mat.shape[0] and h_lambda_mat.shape[1]',
+                self.c_degree+1,'self.c_degree+1',
+                ParameterFormatError
+            )
+            self.h_lambda_mat[:] = h_lambda_mat
 
-        self.degree = self.h_mu_vec.shape[0]-1
-        if self.degree+1 != self.theta_vec.shape[0]:
-            self.theta_vec = np.zeros(self.degree+1)
-            warnings.warn("theta_vec is reinitialized to [0.0, 0.0, ..., 0.0] because dimension of theta_vec and h_parms are mismatched.", ParameterFormatWarning)
+        if h_alpha is not None:
+            self.h_alpha = _check.pos_float(h_alpha,'h_alpha',ParameterFormatError)
+        if h_beta is not None:
+            self.h_beta = _check.pos_float(h_beta,'h_beta',ParameterFormatError)
+
+        return self
 
     def get_h_params(self):
         """Get the hyperparameters of the prior distribution.
@@ -163,27 +134,31 @@ class GenModel(base.Generative):
         """
         self.tau = self.rng.gamma(shape=self.h_alpha,scale=1.0/self.h_beta)
         self.theta_vec = self.rng.multivariate_normal(mean=self.h_mu_vec,cov=np.linalg.inv(self.tau*self.h_lambda_mat))
+        return self
         
-    def set_params(self,theta_vec, tau):
+    def set_params(self,theta_vec=None, tau=None):
         """Set the parameter of the sthocastic data generative model.
 
         Parameters
         ----------
         theta_vec : numpy ndarray, optional
-            a vector of real numbers, by default [0.0, 0.0, ... , 0.0]
-        tau : float, optional
-            a positive real number, by default 1.0
+            a vector of real numbers, by default None
+        tau : float, optional, optional
+            a positive real number, by default None
         """
-        self.theta_vec = _check.float_vec(theta_vec,'theta_vec',ParameterFormatError)
-        self.tau = _check.pos_float(tau,'tau',ParameterFormatError)
+        if theta_vec is not None:
+            _check.float_vec(theta_vec,'theta_vec',ParameterFormatError)
+            _check.shape_consistency(
+                theta_vec.shape[0],'theta_vec.shape[0]',
+                self.c_degree+1,'self.c_degree+1',
+                ParameterFormatError
+            )
+            self.theta_vec[:] = theta_vec
 
-        self.degree = self.theta_vec.shape[0]-1
-        if self.degree+1 != self.h_mu_vec.shape[0]:
-            self.h_mu_vec = np.zeros(self.degree+1)
-            warnings.warn("h_mu_vec is reinitialized to [0.0, 0.0, ..., 0.0] because dimension of h_mu_vec and theta_vec are mismatched.", ParameterFormatWarning)
-        if self.degree+1 != self.h_lambda_mat.shape[0]:
-            self.h_lambda_mat = np.identity(self.degree+1)
-            warnings.warn("h_lambda_mat is reinitialized to the identity matrix because dimension of h_lambda_mat and theta_vec are mismatched.", ParameterFormatWarning)
+        if tau is not None:
+            self.tau = _check.pos_float(tau,'tau',ParameterFormatError)
+
+        return self
 
     def get_params(self):
         """Get the parameter of the sthocastic data generative model.
@@ -203,8 +178,9 @@ class GenModel(base.Generative):
         ----------
         sample_length : int
             A positive integer
-        initial_valules : numpy ndarray
-            1 dimensional float array whose size coincide with ``self.degree``.
+        initial_valules : numpy ndarray, optional
+            1 dimensional float array whose size coincide with ``self.c_degree``, 
+            by default None.
 
         Returns
         -------
@@ -212,19 +188,19 @@ class GenModel(base.Generative):
             1 dimensional float array whose size is ``sammple_length``.
         """
         _check.pos_int(sample_length,'sample_length',DataFormatError)
-        x = np.zeros(sample_length+self.degree)
+        x = np.zeros(sample_length+self.c_degree)
         if initial_values is not None:
             _check.float_vec(initial_values,'initial_values',DataFormatError)
-            if initial_values.shape != (self.degree,):
-                raise(DataFormatError("initial_values must be a 1 dimensional float array whose size coincide with ``self.degree``"))
-            x[:self.degree] = initial_values
+            if initial_values.shape != (self.c_degree,):
+                raise(DataFormatError("initial_values must be a 1 dimensional float array whose size coincide with ``self.c_degree``"))
+            x[:self.c_degree] = initial_values
         
-        _explanatory_vec = np.ones(self.degree+1)
-        for n in range(self.degree,sample_length+self.degree):
-            _explanatory_vec[1:] = x[n-self.degree:n]
+        _explanatory_vec = np.ones(self.c_degree+1)
+        for n in range(self.c_degree,sample_length+self.c_degree):
+            _explanatory_vec[1:] = x[n-self.c_degree:n]
             x[n] = self.rng.normal(loc=self.theta_vec @ _explanatory_vec, scale=1.0/np.sqrt(self.tau))
 
-        return x[self.degree:]
+        return x[self.c_degree:]
         
     def save_sample(self,filename,sample_length,initial_values=None):
         """Save the generated sample as NumPy ``.npz`` format.
@@ -238,8 +214,9 @@ class GenModel(base.Generative):
             ``.npz`` will be appended if it isn't there.
         sample_length : int
             A positive integer
-        initial_valules : numpy ndarray
-            1 dimensional float array whose size coincide with ``self.degree``.
+        initial_valules : numpy ndarray, optional
+            1 dimensional float array whose size coincide with ``self.c_degree``, 
+            by default None.
         
         See Also
         --------
@@ -256,14 +233,15 @@ class GenModel(base.Generative):
             A positive integer, by default 50
         sample_num : int, optional
             A positive integer, by default 5
-        initial_valules : numpy ndarray
-            1 dimensional float array whose size coincide with ``self.degree``.
+        initial_valules : numpy ndarray, optional
+            1 dimensional float array whose size coincide with ``self.c_degree``, 
+            by default None.
         
         Examples
         --------
         >>> import numpy as np
         >>> from bayesml import autoregressive
-        >>> model = autoregressive.GenModel(theta_vec=np.array([0,1]))
+        >>> model = autoregressive.GenModel(c_degree=1,theta_vec=np.array([0,1]))
         >>> model.visualize_model()
         theta_vec:[0,1]
         tau:1.0
@@ -287,98 +265,86 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
 
     Parameters
     ----------
-    degree : int, optional
-        a positive integer. Default is None, in which case 
-        a value consistent with ``theta_vec``, ``h_mu``, 
-        and ``h_lambda_mat`` is used. If all of them are not given,
-        degree is assumed to be 1.
+    c_degree : int
+        a positive integer.
     h0_mu_vec : numpy ndarray, optional
         a vector of real numbers, by default [0.0, 0.0, ... , 0.0]
     h0_lambda_mat : numpy ndarray, optional
-        a positibe definate matrix, by default the identity matrix
+        a positive definate matrix, by default the identity matrix
     h0_alpha : float, optional
         a positive real number, by default 1.0
     h0_beta : float, optional
-        a positibe real number, by default 1.0
+        a positive real number, by default 1.0
 
     Attributes
     ----------
     hn_mu_vec : numpy ndarray
         a vector of real numbers
     hn_lambda_mat : numpy ndarray
-        a positibe definate matrix
+        a positive definate matrix
     hn_alpha : float
         a positive real number
     hn_beta : float
-        a positibe real number
+        a positive real number
     p_m : float
         a positive real number
     p_lambda : float
-        a positibe real number
+        a positive real number
     p_nu : float
-        a positibe real number
+        a positive real number
     """
     def __init__(
-        self,
-        *,
-        degree=None,
-        h0_mu_vec=None,
-        h0_lambda_mat=None,
-        h0_alpha=1.0,
-        h0_beta=1.0,
-        ):
-        if degree is not None:
-            self.degree = _check.nonneg_int(degree,'degree',ParameterFormatError)
-            if h0_mu_vec is None:
-                self.h0_mu_vec = np.zeros(self.degree+1)
-            else:
-                self.h0_mu_vec = _check.float_vec(h0_mu_vec,'h0_mu_vec',ParameterFormatError)
+            self,
+            c_degree,
+            h0_mu_vec=None,
+            h0_lambda_mat=None,
+            h0_alpha=1.0,
+            h0_beta=1.0,
+            ):
+        # constants
+        self.c_degree = _check.nonneg_int(c_degree,'c_degree',ParameterFormatError)
 
-            if h0_lambda_mat is None:
-                self.h0_lambda_mat = np.identity(self.degree+1)
-            else:
-                self.h0_lambda_mat = _check.pos_def_sym_mat(h0_lambda_mat,'h0_lambda_mat',ParameterFormatError)
+        # h0_params
+        self.h0_mu_vec = np.zeros(self.c_degree+1)
+        self.h0_lambda_mat = np.eye(self.c_degree+1)
+        self.h0_alpha = 1.0
+        self.h0_beta = 1.0
 
-        elif h0_mu_vec is not None:
-            self.h0_mu_vec = _check.float_vec(h0_mu_vec,'h0_mu_vec',ParameterFormatError)
-            self.degree = self.h0_mu_vec.shape[0]-1
-            if h0_lambda_mat is None:
-                self.h0_lambda_mat = np.identity(self.degree+1)
-            else:
-                self.h0_lambda_mat = _check.pos_def_sym_mat(h0_lambda_mat,'h0_lambda_mat',ParameterFormatError)
-        
-        elif h0_lambda_mat is not None:
-            self.h0_lambda_mat = _check.pos_def_sym_mat(h0_lambda_mat,'h0_lambda_mat',ParameterFormatError)
-            self.degree = self.h0_lambda_mat.shape[0]-1
-            self.h0_mu_vec = np.zeros(self.degree+1)
-        
-        else:
-            self.degree = 1
-            self.h0_mu_vec = np.zeros(self.degree+1)
-            self.h0_lambda_mat = np.identity(self.degree+1)
+        # hn_params
+        self.hn_mu_vec = np.zeros(self.c_degree+1)
+        self.hn_lambda_mat = np.eye(self.c_degree+1)
+        self.hn_alpha = 1.0
+        self.hn_beta = 1.0
 
-        if (self.degree+1 != self.h0_mu_vec.shape[0]
-            or self.degree+1 != self.h0_lambda_mat.shape[0]):
-                raise(ParameterFormatError(
-                    "(degree+1) and dimensions of h0_mu_vec"
-                    +" and h0_lambda_mat must be the same,"
-                    +" if two or more of them are specified."))
+        # p_params
+        self.p_m = 0.0
+        self.p_lambda = 0.5
+        self.p_nu = 2.0
 
-        self.h0_alpha = _check.pos_float(h0_alpha,'h0_alpha',ParameterFormatError)
-        self.h0_beta = _check.pos_float(h0_beta,'h0_beta',ParameterFormatError)
+        self.set_h0_params(
+            h0_mu_vec,
+            h0_lambda_mat,
+            h0_alpha,
+            h0_beta,
+        )
 
-        self.hn_mu_vec = np.copy(self.h0_mu_vec)
-        self.hn_lambda_mat = np.copy(self.h0_lambda_mat)
-        self.hn_alpha = self.h0_alpha
-        self.hn_beta = self.h0_beta
+    def get_constants(self):
+        """Get constants of LearnModel.
 
-        _explanatory_vec = np.ones(self.degree+1)
-        _explanatory_vec[1:] = 0
-        self.p_m = self.hn_mu_vec @ _explanatory_vec
-        self.p_lambda = self.hn_alpha / self.hn_beta / (1.0 + _explanatory_vec @ np.linalg.solve(self.hn_lambda_mat,_explanatory_vec))
-        self.p_nu = 2.0 * self.hn_alpha
+        Returns
+        -------
+        constants : dict of {str: int}
+            * ``"c_degree"`` : the value of ``self.c_degree``
+        """
+        return {'c_degree':self.c_degree}
 
-    def set_h0_params(self,h0_mu_vec,h0_lambda_mat,h0_alpha,h0_beta):
+    def set_h0_params(
+            self,
+            h0_mu_vec=None,
+            h0_lambda_mat=None,
+            h0_alpha=None,
+            h0_beta=None,
+            ):
         """Set initial values of the hyperparameter of the posterior distribution.
 
         Note that the parameters of the predictive distribution are also calculated from 
@@ -386,26 +352,40 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
 
         Parameters
         ----------
-        h0_mu_vec : numpy ndarray
-            a vector of real numbers
-        h0_lambda_mat : numpy ndarray
-            a positibe definate matrix
-        h0_alpha : float
-            a positive real number
-        h0_beta : float
-            a positibe real number
+        h0_mu_vec : numpy ndarray, optional
+            a vector of real numbers, by default None.
+        h0_lambda_mat : numpy ndarray, optional
+            a positive definate matrix, by default None.
+        h0_alpha : float, optional
+            a positive real number, by default None.
+        h0_beta : float, optional
+            a positive real number, by default None.
         """
-        self.h0_mu_vec = _check.float_vec(h0_mu_vec,'h0_mu_vec',ParameterFormatError)
-        self.h0_lambda_mat = _check.pos_def_sym_mat(h0_lambda_mat,'h0_lambda_mat',ParameterFormatError)
-        self.h0_alpha = _check.pos_float(h0_alpha,'h0_alpha',ParameterFormatError)
-        self.h0_beta = _check.pos_float(h0_beta,'h0_beta',ParameterFormatError)
+        if h0_mu_vec is not None:
+            _check.float_vec(h0_mu_vec,'h0_mu_vec',ParameterFormatError)
+            _check.shape_consistency(
+                h0_mu_vec.shape[0],'h0_mu_vec.shape[0]',
+                self.c_degree+1,'self.c_degree+1',
+                ParameterFormatError
+            )
+            self.h0_mu_vec[:] = h0_mu_vec
 
-        self.degree = self.h0_mu_vec.shape[0]-1
-        if (self.h0_mu_vec.shape[0] != self.h0_lambda_mat.shape[0]):
-                raise(ParameterFormatError(
-                    "dimensions of h0_mu_vec and h0_lambda_mat must be the same"))
+        if h0_lambda_mat is not None:
+            _check.pos_def_sym_mat(h0_lambda_mat,'h0_lambda_mat',ParameterFormatError)
+            _check.shape_consistency(
+                h0_lambda_mat.shape[0],'h0_lambda_mat.shape[0] and h0_lambda_mat.shape[1]',
+                self.c_degree+1,'self.c_degree+1',
+                ParameterFormatError
+            )
+            self.h0_lambda_mat[:] = h0_lambda_mat
+
+        if h0_alpha is not None:
+            self.h0_alpha = _check.pos_float(h0_alpha,'h0_alpha',ParameterFormatError)
+        if h0_beta is not None:
+            self.h0_beta = _check.pos_float(h0_beta,'h0_beta',ParameterFormatError)
 
         self.reset_hn_params()
+        return self
 
     def get_h0_params(self):
         """Get the initial values of the hyperparameters of the posterior distribution.
@@ -433,7 +413,13 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         """
         return {"hn_mu_vec":self.hn_mu_vec, "hn_lambda_mat":self.hn_lambda_mat, "hn_alpha":self.hn_alpha, "hn_beta":self.hn_beta}
     
-    def set_hn_params(self,hn_mu_vec,hn_lambda_mat,hn_alpha,hn_beta):
+    def set_hn_params(
+            self,
+            hn_mu_vec=None,
+            hn_lambda_mat=None,
+            hn_alpha=None,
+            hn_beta=None,
+            ):
         """Set updated values of the hyperparameter of the posterior distribution.
 
         Note that the parameters of the predictive distribution are also calculated from 
@@ -441,53 +427,40 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
 
         Parameters
         ----------
-        hn_mu_vec : numpy ndarray
-            a vector of real numbers
-        hn_lambda_mat : numpy ndarray
-            a positibe definate matrix
-        hn_alpha : float
-            a positive real number
-        hn_beta : float
-            a positibe real number
+        hn_mu_vec : numpy ndarray, optional
+            a vector of real numbers, by default None.
+        hn_lambda_mat : numpy ndarray, optional
+            a positive definate matrix, by default None.
+        hn_alpha : float, optional
+            a positive real number, by default None.
+        hn_beta : float, optional
+            a positive real number, by default None.
         """
-        self.hn_mu_vec = _check.float_vec(hn_mu_vec,'hn_mu_vec',ParameterFormatError)
-        self.hn_lambda_mat = _check.pos_def_sym_mat(hn_lambda_mat,'hn_lambda_mat',ParameterFormatError)
-        self.hn_alpha = _check.pos_float(hn_alpha,'hn_alpha',ParameterFormatError)
-        self.hn_beta = _check.pos_float(hn_beta,'hn_beta',ParameterFormatError)
+        if hn_mu_vec is not None:
+            _check.float_vec(hn_mu_vec,'hn_mu_vec',ParameterFormatError)
+            _check.shape_consistency(
+                hn_mu_vec.shape[0],'hn_mu_vec.shape[0]',
+                self.c_degree+1,'self.c_degree+1',
+                ParameterFormatError
+            )
+            self.hn_mu_vec[:] = hn_mu_vec
 
-        self.degree = self.hn_mu_vec.shape[0]-1
-        if (self.hn_mu_vec.shape[0] != self.hn_lambda_mat.shape[0]):
-                raise(ParameterFormatError(
-                    "dimensions of hn_mu_vec and hn_lambda_mat must be the same"))
-        self.calc_pred_dist(np.zeros(self.degree))
+        if hn_lambda_mat is not None:
+            _check.pos_def_sym_mat(hn_lambda_mat,'hn_lambda_mat',ParameterFormatError)
+            _check.shape_consistency(
+                hn_lambda_mat.shape[0],'hn_lambda_mat.shape[0] and hn_lambda_mat.shape[1]',
+                self.c_degree+1,'self.c_degree+1',
+                ParameterFormatError
+            )
+            self.hn_lambda_mat[:] = hn_lambda_mat
 
-    def overwrite_h0_params(self):
-        """Overwrite the initial values of the hyperparameters of the posterior distribution by the learned values.
-        
-        They are overwritten by ``self.hn_mu_vec``, ``slef.hn_lambda_mat``, ``self.hn_alpha`` and ``self.hn_beta``.
-        Note that the parameters of the predictive distribution are also calculated 
-        from ``self.hn_mu_vec``, ``slef.hn_lambda_mat``, ``self.hn_alpha`` and ``self.hn_beta``.
-        """
-        self.h0_mu_vec = np.copy(self.hn_mu_vec)
-        self.h0_lambda_mat = np.copy(self.hn_lambda_mat)
-        self.h0_alpha = self.hn_alpha
-        self.h0_beta = self.hn_beta
+        if hn_alpha is not None:
+            self.hn_alpha = _check.pos_float(hn_alpha,'hn_alpha',ParameterFormatError)
+        if hn_beta is not None:
+            self.hn_beta = _check.pos_float(hn_beta,'hn_beta',ParameterFormatError)
 
-        self.calc_pred_dist(np.zeros(self.degree))
-
-    def reset_hn_params(self):
-        """Reset the hyperparameters of the posterior distribution to their initial values.
-        
-        They are reset to ``self.h0_mu_vec``, ``slef.h0_lambda_mat``, ``self.h0_alpha`` and ``self.h0_beta``.
-        Note that the parameters of the predictive distribution are also calculated 
-        from ``self.h0_mu_vec``, ``slef.h0_lambda_mat``, ``self.h0_alpha`` and ``self.h0_beta``.
-        """
-        self.hn_mu_vec = np.copy(self.h0_mu_vec)
-        self.hn_lambda_mat = np.copy(self.h0_lambda_mat)
-        self.hn_alpha = self.h0_alpha
-        self.hn_beta = self.h0_beta
-
-        self.calc_pred_dist(np.zeros(self.degree))
+        self.calc_pred_dist(np.zeros(self.c_degree))
+        return self
 
     def update_posterior(self,x,padding=None):
         """Update the hyperparameters of the posterior distribution using traning data.
@@ -498,21 +471,21 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
             1 dimensional float array
         padding : str, optional
             Padding option for data values at negative time points.
-            Default is ``None``, in which case the first ``self.degree`` values of ``X`` are used as initial values.
+            Default is ``None``, in which case the first ``self.c_degree`` values of ``X`` are used as initial values.
             If \"zeros\" is given, the zero vector is used as a initial value.
         """
         _check.float_vec(x,'x',DataFormatError)
-        if x.shape[0] <= self.degree:
-            raise(DataFormatError("The length of x must greater than self.degree"))
-        x_mat = np.zeros((x.shape[0],self.degree+1))
+        if x.shape[0] <= self.c_degree:
+            raise(DataFormatError("The length of x must greater than self.c_degree"))
+        x_mat = np.zeros((x.shape[0],self.c_degree+1))
         x_mat[:,0] = 1.0
-        for n in range(1,self.degree+1):
+        for n in range(1,self.c_degree+1):
             x_mat[n,-n:] = x[:n]
-        for n in range(self.degree+1,x.shape[0]):
-            x_mat[n,1:] = x[n-self.degree:n]
+        for n in range(self.c_degree+1,x.shape[0]):
+            x_mat[n,1:] = x[n-self.c_degree:n]
         
-        mu_tmp = np.copy(self.hn_mu_vec)
-        lambda_tmp = np.copy(self.hn_lambda_mat)
+        mu_tmp = np.array(self.hn_mu_vec)
+        lambda_tmp = np.array(self.hn_lambda_mat)
         if padding == "zeros":
             self.hn_lambda_mat += x_mat.T @ x_mat
             self.hn_mu_vec[:] = np.linalg.solve(self.hn_lambda_mat,
@@ -521,12 +494,13 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
             self.hn_beta += (-self.hn_mu_vec[np.newaxis,:] @ self.hn_lambda_mat @ self.hn_mu_vec[:,np.newaxis]
                             + x @ x + mu_tmp[np.newaxis,:] @ lambda_tmp @ mu_tmp[:,np.newaxis])[0,0] / 2.0
         else:
-            self.hn_lambda_mat += x_mat[self.degree:].T @ x_mat[self.degree:]
+            self.hn_lambda_mat += x_mat[self.c_degree:].T @ x_mat[self.c_degree:]
             self.hn_mu_vec[:] = np.linalg.solve(self.hn_lambda_mat,
-                                                lambda_tmp @ mu_tmp[:,np.newaxis] + x_mat[self.degree:].T @ x[self.degree:,np.newaxis])[:,0]
-            self.hn_alpha += (x.shape[0]-self.degree) / 2.0
+                                                lambda_tmp @ mu_tmp[:,np.newaxis] + x_mat[self.c_degree:].T @ x[self.c_degree:,np.newaxis])[:,0]
+            self.hn_alpha += (x.shape[0]-self.c_degree) / 2.0
             self.hn_beta += (-self.hn_mu_vec[np.newaxis,:] @ self.hn_lambda_mat @ self.hn_mu_vec[:,np.newaxis] 
-                            + x[self.degree:] @ x[self.degree:] + mu_tmp[np.newaxis,:] @ lambda_tmp @ mu_tmp[:,np.newaxis])[0,0] / 2.0
+                            + x[self.c_degree:] @ x[self.c_degree:] + mu_tmp[np.newaxis,:] @ lambda_tmp @ mu_tmp[:,np.newaxis])[0,0] / 2.0
+        return self
 
     def estimate_params(self,loss="squared"):
         """Estimate the parameter of the stochastic data generative model under the given criterion.
@@ -578,7 +552,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         Examples
         --------
         >>> from bayesml import autoregressive
-        >>> gen_model = autoregressive.GenModel(theta_vec=np.array([0,1]),tau=1.0)
+        >>> gen_model = autoregressive.GenModel(c_degree=1,theta_vec=np.array([0,1]),tau=1.0)
         >>> x = gen_model.gen_sample(50)
         >>> learn_model = autoregressive.LearnModel()
         >>> learn_model.update_posterior(x)
@@ -586,8 +560,8 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         
         .. image:: ./images/autoregressive_posterior.png
         """
-        if self.degree != 1:
-            raise(ParameterFormatError("if self.degree != 1, it is impossible to visualize posterior by this function."))
+        if self.c_degree != 1:
+            raise(ParameterFormatError("if self.c_degree != 1, it is impossible to visualize posterior by this function."))
         theta_vec_pdf, tau_pdf = self.estimate_params(loss="KL")
         hn_lambda_mat_inv = np.linalg.inv(self.hn_lambda_mat)
 
@@ -637,16 +611,17 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         Parameters
         ----------
         x : numpy ndarray
-            1 dimensional float array whose size is ``self.degree``
+            1 dimensional float array whose size is ``self.c_degree``
         """
         _check.float_vec(x,'x',DataFormatError)
-        if x.shape != (self.degree,):
-            raise(DataFormatError("x must be a 1 dimensional float array whose size coincide with ``self.degree``"))
-        _explanatory_vec = np.ones(self.degree+1)
+        if x.shape != (self.c_degree,):
+            raise(DataFormatError("x must be a 1 dimensional float array whose size coincide with ``self.c_degree``"))
+        _explanatory_vec = np.ones(self.c_degree+1)
         _explanatory_vec[1:] = x
         self.p_m = self.hn_mu_vec @ _explanatory_vec
         self.p_lambda = self.hn_alpha / self.hn_beta / (1.0 + _explanatory_vec @ np.linalg.solve(self.hn_lambda_mat,_explanatory_vec))
         self.p_nu = 2.0 * self.hn_alpha
+        return self
 
     def make_prediction(self,loss="squared"):
         """Predict a new data point under the given criterion.
@@ -699,8 +674,8 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         Parameters
         ----------
         x : numpy ndarray
-            1 dimensional float array whose size is ``self.degree + 1``,
-            which consists of the ``self.degree`` number of past values
+            1 dimensional float array whose size is ``self.c_degree + 1``,
+            which consists of the ``self.c_degree`` number of past values
             and the current value.
         loss : str, optional
             Loss function underlying the Bayes risk function, by default \"squared\".
@@ -714,9 +689,9 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
             as rv_frozen object of scipy.stats.
         """
         _check.float_vec(x,'x',DataFormatError)
-        if x.shape != (self.degree+1,):
-            raise(DataFormatError("x must be a 1 dimensional float array whose size coincide with ``self.degree + 1``"))
-        self.calc_pred_dist(x[:self.degree])
+        if x.shape != (self.c_degree+1,):
+            raise(DataFormatError("x must be a 1 dimensional float array whose size coincide with ``self.c_degree + 1``"))
+        self.calc_pred_dist(x[:self.c_degree])
         prediction = self.make_prediction(loss=loss)
         self.update_posterior(x)
         return prediction
