@@ -48,7 +48,6 @@ class GenModel(base.Generative):
             self,
             c_num_classes,
             c_degree,
-            *,
             pi_vec=None,
             mu_vecs=None,
             lambda_mats=None,
@@ -67,14 +66,14 @@ class GenModel(base.Generative):
         # params
         self.pi_vec = np.ones(self.c_num_classes) / self.c_num_classes
         self.mu_vecs = np.zeros([self.c_num_classes,self.c_degree])
-        self.lambda_mats = np.tile(np.identity(self.c_degree),[self.c_num_classes,1,1])
+        self.lambda_mats = np.tile(np.eye(self.c_degree),[self.c_num_classes,1,1])
 
         # h_params
         self.h_alpha_vec = np.ones(self.c_num_classes) / 2        
         self.h_m_vecs = np.zeros([self.c_num_classes,self.c_degree])
         self.h_kappas = np.ones(self.c_num_classes)
         self.h_nus = np.ones(self.c_num_classes) * self.c_degree
-        self.h_w_mats = np.tile(np.identity(self.c_degree),[self.c_num_classes,1,1])
+        self.h_w_mats = np.tile(np.eye(self.c_degree),[self.c_num_classes,1,1])
         
         self.set_params(
             pi_vec,
@@ -88,7 +87,18 @@ class GenModel(base.Generative):
             h_nus,
             h_w_mats,
             )
-        
+
+    def get_constants(self):
+        """Get constants of GenModel.
+
+        Returns
+        -------
+        constants : dict of {str: int, numpy.ndarray}
+            * ``"c_num_classes"`` : the value of ``self.c_num_classes``
+            * ``"c_degree"`` : the value of ``self.c_degree``
+        """
+        return {"c_num_classes":self.c_num_classes, "c_degree":self.c_degree}
+
     def set_h_params(
             self,
             h_alpha_vec=None,
@@ -143,6 +153,7 @@ class GenModel(base.Generative):
                     "h_w_mats.shape[-1] and h_w_mats.shape[-2] must coincide with self.c_degree: "
                     +f"h_w_mats.shape[-1] and h_w_mats.shape[-2] = {h_w_mats.shape[-1]}, self.c_degree = {self.c_degree}"))
             self.h_w_mats[:] = h_w_mats
+        return self
 
     def get_h_params(self):
         """Get the hyperparameters of the prior distribution.
@@ -171,7 +182,8 @@ class GenModel(base.Generative):
         for k in range(self.c_num_classes):
             self.lambda_mats[k] = ss_wishart.rvs(df=self.h_nus[k],scale=self.h_w_mats[k],random_state=self.rng)
             self.mu_vecs[k] = self.rng.multivariate_normal(mean=self.h_m_vecs[k],cov=np.linalg.inv(self.h_kappas[k]*self.lambda_mats[k]))
-    
+        return self
+
     def set_params(
             self,
             pi_vec=None,
@@ -212,6 +224,7 @@ class GenModel(base.Generative):
                     "lambda_mats.shape[-1] and lambda_mats.shape[-2] must coincide with self.c_degree:"
                     +f"lambda_mats.shape[-1] and lambda_mats.shape[-2] = {lambda_mats.shape[-1]}, self.c_degree = {self.c_degree}"))
             self.lambda_mats[:] = lambda_mats
+        return self
 
     def get_params(self):
         """Get the parameter of the sthocastic data generative model.
@@ -409,7 +422,6 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
             self,
             c_num_classes,
             c_degree,
-            *,
             h0_alpha_vec = None,
             h0_m_vecs = None,
             h0_kappas = None,
@@ -427,7 +439,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         self.h0_m_vecs = np.zeros([self.c_num_classes,self.c_degree])
         self.h0_kappas = np.ones(self.c_num_classes)
         self.h0_nus = np.ones(self.c_num_classes) * self.c_degree
-        self.h0_w_mats = np.tile(np.identity(self.c_degree),[self.c_num_classes,1,1])
+        self.h0_w_mats = np.tile(np.eye(self.c_degree),[self.c_num_classes,1,1])
         self.h0_w_mats_inv = np.linalg.inv(self.h0_w_mats)
 
         self._ln_c_h0_alpha = 0.0
@@ -477,6 +489,17 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
             h0_w_mats,
         )
 
+    def get_constants(self):
+        """Get constants of LearnModel.
+
+        Returns
+        -------
+        constants : dict of {str: int, numpy.ndarray}
+            * ``"c_num_classes"`` : the value of ``self.c_num_classes``
+            * ``"c_degree"`` : the value of ``self.c_degree``
+        """
+        return {"c_num_classes":self.c_num_classes, "c_degree":self.c_degree}
+
     def set_h0_params(
             self,
             h0_alpha_vec = None,
@@ -535,6 +558,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
 
         self._calc_prior_char()
         self.reset_hn_params()
+        return self
 
     def get_h0_params(self):
         """Get the initial values of the hyperparameters of the posterior distribution.
@@ -614,6 +638,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         self._calc_q_lambda_char()
 
         self.calc_pred_dist()
+        return self
 
     def get_hn_params(self):
         """Get the hyperparameters of the posterior distribution.
@@ -699,11 +724,19 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
 
     def _calc_n_x_bar_s(self,x):
         self.ns[:] = self.r_vecs.sum(axis=0)
-        self.x_bar_vecs[:] = (self.r_vecs[:,:,np.newaxis] * x[:,np.newaxis,:]).sum(axis=0) / self.ns[:,np.newaxis]
-        self.s_mats[:] = np.sum(self.r_vecs[:,:,np.newaxis,np.newaxis]
-                                * ((x[:,np.newaxis,:] - self.x_bar_vecs)[:,:,:,np.newaxis]
-                                   @ (x[:,np.newaxis,:] - self.x_bar_vecs)[:,:,np.newaxis,:]),
-                                axis=0) / self.ns[:,np.newaxis,np.newaxis]
+        indices = self.ns.astype(bool)
+        if np.all(indices):
+            self.x_bar_vecs[:] = (self.r_vecs[:,:,np.newaxis] * x[:,np.newaxis,:]).sum(axis=0) / self.ns[:,np.newaxis]
+            self.s_mats[:] = np.sum(self.r_vecs[:,:,np.newaxis,np.newaxis]
+                                    * ((x[:,np.newaxis,:] - self.x_bar_vecs)[:,:,:,np.newaxis]
+                                    @ (x[:,np.newaxis,:] - self.x_bar_vecs)[:,:,np.newaxis,:]),
+                                    axis=0) / self.ns[:,np.newaxis,np.newaxis]
+        else:
+            self.x_bar_vecs[indices] = (self.r_vecs[:,indices,np.newaxis] * x[:,np.newaxis,:]).sum(axis=0) / self.ns[indices,np.newaxis]
+            self.s_mats[indices] = np.sum(self.r_vecs[:,indices,np.newaxis,np.newaxis]
+                                    * ((x[:,np.newaxis,:] - self.x_bar_vecs[indices])[:,:,:,np.newaxis]
+                                    @ (x[:,np.newaxis,:] - self.x_bar_vecs[indices])[:,:,np.newaxis,:]),
+                                    axis=0) / self.ns[indices,np.newaxis,np.newaxis]
 
     def _init_random_responsibility(self,x):
         self.r_vecs[:] = self.rng.dirichlet(np.ones(self.c_num_classes),self.r_vecs.shape[0])
@@ -766,7 +799,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
             self.hn_w_mats_inv[k] = ((_subsample - self.hn_m_vecs[k]).T
                                  @ (_subsample - self.hn_m_vecs[k])
                                  / _size * self.hn_nus[k]
-                                 + np.identity(self.c_degree) * 1.0E-5) # avoid singular matrix
+                                 + np.eye(self.c_degree) * 1.0E-5) # avoid singular matrix
             self.hn_w_mats[k] = np.linalg.inv(self.hn_w_mats_inv[k])
         self._calc_q_lambda_char()
 
@@ -810,12 +843,12 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         self.r_vecs = np.empty([x.shape[0],self.c_num_classes])
 
         tmp_vl = 0.0
-        tmp_alpha_vec = np.copy(self.hn_alpha_vec)
-        tmp_m_vecs = np.copy(self.hn_m_vecs)
-        tmp_kappas = np.copy(self.hn_kappas)
-        tmp_nus = np.copy(self.hn_nus)
-        tmp_w_mats = np.copy(self.hn_w_mats)
-        tmp_w_mats_inv = np.copy(self.hn_w_mats_inv)
+        tmp_alpha_vec = np.array(self.hn_alpha_vec)
+        tmp_m_vecs = np.array(self.hn_m_vecs)
+        tmp_kappas = np.array(self.hn_kappas)
+        tmp_nus = np.array(self.hn_nus)
+        tmp_w_mats = np.array(self.hn_w_mats)
+        tmp_w_mats_inv = np.array(self.hn_w_mats_inv)
 
         convergence_flag = True
         for i in range(num_init):
@@ -867,6 +900,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         self._calc_q_pi_char()
         self._calc_q_lambda_char()
         self._update_q_z(x)
+        return self
 
     def estimate_params(self,loss="squared"):
         """Estimate the parameter of the stochastic data generative model under the given criterion.
@@ -907,7 +941,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
             if np.all(self.hn_alpha_vec > 1):
                 pi_vec_hat[:] = (self.hn_alpha_vec - 1) / (np.sum(self.hn_alpha_vec) - self.c_degree)
             else:
-                warnings.warn("MAP estimate of lambda_mat doesn't exist for the current hn_alpha_vec.",ResultWarning)
+                warnings.warn("MAP estimate of pi_vec doesn't exist for the current hn_alpha_vec.",ResultWarning)
                 pi_vec_hat[:] = np.nan
 
             lambda_mats_hat = np.empty([self.c_num_classes,self.c_degree,self.c_degree])
@@ -1040,6 +1074,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         self.p_mu_vecs[:] = self.hn_m_vecs
         self.p_nus[:] = self.hn_nus - self.c_degree + 1
         self.p_lambda_mats[:] = (self.hn_kappas * self.p_nus / (self.hn_kappas + 1))[:,np.newaxis,np.newaxis] * self.hn_w_mats
+        return self
 
     def make_prediction(self,loss="squared"):
         """Predict a new data point under the given criterion.
@@ -1157,7 +1192,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         if loss == "squared":
             return self.r_vecs
         elif loss == "0-1":
-            return np.identity(self.c_num_classes,dtype=int)[np.argmax(self.r_vecs,axis=1)]
+            return np.eye(self.c_num_classes,dtype=int)[np.argmax(self.r_vecs,axis=1)]
         elif loss == "KL":
             return self.r_vecs
         else:
