@@ -95,7 +95,7 @@ class GenModel(base.Generative):
         self.pi_vec = np.ones(self.c_num_classes) / self.c_num_classes
         self.a_mat = np.ones([self.c_num_classes,self.c_num_classes]) / self.c_num_classes
         self.mu_vecs = np.zeros([self.c_num_classes,self.c_degree])
-        self.lambda_mats = np.tile(np.identity(self.c_degree),[self.c_num_classes,1,1])
+        self.lambda_mats = np.tile(np.eye(self.c_degree),[self.c_num_classes,1,1])
 
         # h_params
         self.h_eta_vec = np.ones(self.c_num_classes) / 2.0
@@ -103,7 +103,7 @@ class GenModel(base.Generative):
         self.h_m_vecs = np.zeros([self.c_num_classes,self.c_degree])
         self.h_kappas = np.ones([self.c_num_classes])
         self.h_nus = np.ones(self.c_num_classes) * self.c_degree
-        self.h_w_mats = np.tile(np.identity(self.c_degree),[self.c_num_classes,1,1])
+        self.h_w_mats = np.tile(np.eye(self.c_degree),[self.c_num_classes,1,1])
 
         self.set_params(
             pi_vec,
@@ -119,6 +119,17 @@ class GenModel(base.Generative):
             h_nus,
             h_w_mats)
 
+    def get_constants(self):
+        """Get constants of GenModel.
+
+        Returns
+        -------
+        constants : dict of {str: int, numpy.ndarray}
+            * ``"c_num_classes"`` : the value of ``self.c_num_classes``
+            * ``"c_degree"`` : the value of ``self.c_degree``
+        """
+        return {"c_num_classes":self.c_num_classes, "c_degree":self.c_degree}
+
     def set_params(
             self,
             pi_vec=None,
@@ -184,6 +195,7 @@ class GenModel(base.Generative):
                 ParameterFormatError
                 )
             self.lambda_mats[:] = lambda_mats
+        return self
 
     def set_h_params(
             self,
@@ -259,6 +271,7 @@ class GenModel(base.Generative):
                 ParameterFormatError
                 )
             self.h_w_mats[:] = h_w_mats
+        return self
 
     def get_params(self):
         """Get the parameter of the sthocastic data generative model.
@@ -307,6 +320,7 @@ class GenModel(base.Generative):
         for k in range(self.c_num_classes):
             self.lambda_mats[k] = ss_wishart.rvs(df=self.h_nus[k],scale=self.h_w_mats[k],random_state=self.rng)
             self.mu_vecs[k] = self.rng.multivariate_normal(mean=self.h_m_vecs[k],cov=np.linalg.inv(self.h_kappas[k]*self.lambda_mats[k]))
+        return self
 
     def gen_sample(self,sample_length):
         """Generate a sample from the stochastic data generative model.
@@ -520,7 +534,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         self.h0_m_vecs = np.zeros([self.c_num_classes,self.c_degree])
         self.h0_kappas = np.ones([self.c_num_classes])
         self.h0_nus = np.ones(self.c_num_classes) * self.c_degree
-        self.h0_w_mats = np.tile(np.identity(self.c_degree),[self.c_num_classes,1,1])
+        self.h0_w_mats = np.tile(np.eye(self.c_degree),[self.c_num_classes,1,1])
         self.h0_w_mats_inv = np.linalg.inv(self.h0_w_mats)
 
         self._ln_c_h0_eta_vec = 0.0
@@ -591,6 +605,17 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
             h0_w_mats,
         )
 
+    def get_constants(self):
+        """Get constants of LearnModel.
+
+        Returns
+        -------
+        constants : dict of {str: int, numpy.ndarray}
+            * ``"c_num_classes"`` : the value of ``self.c_num_classes``
+            * ``"c_degree"`` : the value of ``self.c_degree``
+        """
+        return {"c_num_classes":self.c_num_classes, "c_degree":self.c_degree}
+
     def set_h0_params(
             self,
             h0_eta_vec = None,
@@ -671,6 +696,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
 
         self._calc_prior_char()
         self.reset_hn_params()
+        return self
 
     def get_h0_params(self):
         """Get the hyperparameters of the prior distribution.
@@ -775,6 +801,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         self._calc_q_lambda_char()
 
         self.calc_pred_dist()
+        return self
 
     def get_hn_params(self):
         """Get the hyperparameters of the posterior distribution.
@@ -810,23 +837,27 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
     def _calc_n_m_x_bar_s(self,x):
         self.ns[:] = self.gamma_vecs.sum(axis=0)
         self.ms[:] = self.xi_mats.sum(axis=0) # xi must be initialized as a zero matrix
-        self.x_bar_vecs[:] = (self.gamma_vecs[:,:,np.newaxis] * x[:,np.newaxis,:]).sum(axis=0) / self.ns[:,np.newaxis]
-        self.s_mats[:] = np.sum(self.gamma_vecs[:,:,np.newaxis,np.newaxis]
-                                * ((x[:,np.newaxis,:] - self.x_bar_vecs)[:,:,:,np.newaxis]
-                                   @ (x[:,np.newaxis,:] - self.x_bar_vecs)[:,:,np.newaxis,:]),
-                                axis=0) / self.ns[:,np.newaxis,np.newaxis]
+        indices = self.ns.astype(bool)
+        if np.all(indices):
+            self.x_bar_vecs[:] = (self.gamma_vecs[:,:,np.newaxis] * x[:,np.newaxis,:]).sum(axis=0) / self.ns[:,np.newaxis]
+            self.s_mats[:] = np.sum(self.gamma_vecs[:,:,np.newaxis,np.newaxis]
+                                    * ((x[:,np.newaxis,:] - self.x_bar_vecs)[:,:,:,np.newaxis]
+                                       @ (x[:,np.newaxis,:] - self.x_bar_vecs)[:,:,np.newaxis,:]),
+                                    axis=0) / self.ns[:,np.newaxis,np.newaxis]
+        else:
+            self.x_bar_vecs[indices] = (self.gamma_vecs[:,indices,np.newaxis] * x[:,np.newaxis,:]).sum(axis=0) / self.ns[indices,np.newaxis]
+            self.s_mats[indices] = np.sum(self.gamma_vecs[:,indices,np.newaxis,np.newaxis]
+                                    * ((x[:,np.newaxis,:] - self.x_bar_vecs[indices])[:,:,:,np.newaxis]
+                                       @ (x[:,np.newaxis,:] - self.x_bar_vecs[indices])[:,:,np.newaxis,:]),
+                                    axis=0) / self.ns[indices,np.newaxis,np.newaxis]
 
     def _calc_q_pi_char(self):
         self._ln_pi_tilde_vec[:] = digamma(self.hn_eta_vec) - digamma(self.hn_eta_vec.sum())
-        self._pi_tilde_vec[:] = np.exp(self._ln_pi_tilde_vec)
-        # self._pi_tilde_vec[:] = np.exp(self._ln_pi_tilde_vec - self._ln_pi_tilde_vec.max())
-        # self._pi_tilde_vec[:] /= self._pi_tilde_vec.sum()
+        self._pi_tilde_vec[:] = np.exp(self._ln_pi_tilde_vec - self._ln_pi_tilde_vec.max())
 
     def _calc_q_a_char(self):
         self._ln_a_tilde_mat[:] = digamma(self.hn_zeta_vecs) - digamma(self.hn_zeta_vecs.sum(axis=1,keepdims=True))
-        self._a_tilde_mat[:] = np.exp(self._ln_a_tilde_mat)
-        # self._a_tilde_mat[:] = np.exp(self._ln_a_tilde_mat - self._ln_a_tilde_mat.max(axis=1,keepdims=True))
-        # self._a_tilde_mat[:] /= self._a_tilde_mat.sum(axis=1,keepdims=True)
+        self._a_tilde_mat[:] = np.exp(self._ln_a_tilde_mat - self._ln_a_tilde_mat.max())
         self._ln_c_hn_zeta_vecs_sum = np.sum(gammaln(self.hn_zeta_vecs.sum(axis=1)) - gammaln(self.hn_zeta_vecs).sum(axis=1))
 
     def _calc_q_lambda_char(self):
@@ -879,8 +910,8 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
 
         # E[ln q(Z|pi)]
         self._vl_q_z = (-(self.gamma_vecs * self._ln_rho).sum()
-                        -(self.ms * self._ln_a_tilde_mat).sum()
-                        -(self.gamma_vecs[0] * self._ln_pi_tilde_vec).sum()
+                        -(self.ms * (self._ln_a_tilde_mat - self._ln_a_tilde_mat.max())).sum()
+                        -(self.gamma_vecs[0] * (self._ln_pi_tilde_vec - self._ln_pi_tilde_vec.max())).sum()
                         +np.log(self._cs).sum())
 
         # E[ln q(pi)]
@@ -935,7 +966,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
             self.hn_w_mats_inv[k] = ((_subsample - self.hn_m_vecs[k]).T
                                  @ (_subsample - self.hn_m_vecs[k])
                                  / _size * self.hn_nus[k]
-                                 + np.identity(self.c_degree) * 1.0E-5) # avoid singular matrix
+                                 + np.eye(self.c_degree) * 1.0E-5) # avoid singular matrix
             self.hn_w_mats[k] = np.linalg.inv(self.hn_w_mats_inv[k])
         self._calc_q_lambda_char()
 
@@ -1046,13 +1077,13 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         self._cs = np.ones([self._length])
 
         tmp_vl = 0.0
-        tmp_eta_vec = np.copy(self.hn_eta_vec)
-        tmp_zeta_vecs = np.copy(self.hn_zeta_vecs)
-        tmp_m_vecs = np.copy(self.hn_m_vecs)
-        tmp_kappas = np.copy(self.hn_kappas)
-        tmp_nus = np.copy(self.hn_nus)
-        tmp_w_mats = np.copy(self.hn_w_mats)
-        tmp_w_mats_inv = np.copy(self.hn_w_mats_inv)
+        tmp_eta_vec = np.array(self.hn_eta_vec)
+        tmp_zeta_vecs = np.array(self.hn_zeta_vecs)
+        tmp_m_vecs = np.array(self.hn_m_vecs)
+        tmp_kappas = np.array(self.hn_kappas)
+        tmp_nus = np.array(self.hn_nus)
+        tmp_w_mats = np.array(self.hn_w_mats)
+        tmp_w_mats_inv = np.array(self.hn_w_mats_inv)
 
         convergence_flag = True
         for i in range(num_init):
@@ -1108,6 +1139,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         self._calc_q_a_char()
         self._calc_q_lambda_char()
         self._update_q_z(x)
+        return self
 
     def estimate_params(self,loss="squared"):
         """Estimate the parameter under the given criterion.
@@ -1311,6 +1343,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         self.p_mu_vecs[:] = self.hn_m_vecs
         self.p_nus[:] = self.hn_nus - self.c_degree + 1
         self.p_lambda_mats[:] = (self.hn_kappas * self.p_nus / (self.hn_kappas + 1))[:,np.newaxis,np.newaxis] * self.hn_w_mats
+        return self
 
     def make_prediction(self,loss="squared"):
         """Predict a new data point under the given criterion.
@@ -1468,7 +1501,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
             if loss == "squared" or loss == "KL":
                 return self.gamma_vecs
             elif loss == "0-1":
-                return np.identity(self.c_num_classes,dtype=int)[np.argmax(self.gamma_vecs,axis=1)]
+                return np.eye(self.c_num_classes,dtype=int)[np.argmax(self.gamma_vecs,axis=1)]
             else:
                 raise(CriteriaError(f"loss=\"{loss}\" is unsupported. "
                                     +"When viterbi == False, This function supports \"squared\", \"0-1\", and \"KL\"."))
