@@ -22,11 +22,8 @@ class GenModel(base.Generative):
 
     Parameters
     ----------
-    degree : int, optional
-        a positive integer. Default is None, in which case 
-        a value consistent with ``mu_vec``, ``lambda_mat``, 
-        ``h_m_vec``, ``h_w_mat``, and ``h_nu` is used. 
-        If all of them are not given, degree is assumed to be 2.
+    c_degree : int
+        a positive integer.
     mu_vec : numpy.ndarray, optional
         a vector of real numbers, by default [0.0, 0.0, ... , 0.0]
     lambda_mat : numpy.ndarray, optional
@@ -36,7 +33,7 @@ class GenModel(base.Generative):
     h_kappa : float, optional
         a positive real number, by default 1.0
     h_nu : float, optional
-        a real number > degree-1, by default the value of ``degree``
+        a real number > c_degree-1, by default the value of ``c_degree``
     h_w_mat : numpy.ndarray, optional
         a positive definite symetric matrix, by default the identity matrix
     seed : {None, int}, optional
@@ -44,180 +41,86 @@ class GenModel(base.Generative):
         by default None
     """
     def __init__(
-        self,
-        *,
-        degree=None,
-        mu_vec=None,
-        lambda_mat=None,
-        h_m_vec=None,
-        h_kappa=1.0,
-        h_nu=None,
-        h_w_mat=None,
-        seed=None
-        ):
-        
-        if degree is not None:
-            self.degree = _check.pos_int(degree,'degree',ParameterFormatError)
-            if mu_vec is None:
-                self.mu_vec = np.zeros(self.degree)
-            else:
-                self.mu_vec = _check.float_vec(mu_vec,'mu_vec',ParameterFormatError)
-
-            if lambda_mat is None:
-                self.lambda_mat = np.identity(self.degree)
-            else:
-                self.lambda_mat = _check.pos_def_sym_mat(lambda_mat,'lambda_mat',ParameterFormatError)
-
-            if h_m_vec is None:
-                self.h_m_vec = np.zeros(self.degree)
-            else:
-                self.h_m_vec = _check.float_vec(h_m_vec,'h_m_vec',ParameterFormatError)
-
-            if h_w_mat is None:
-                self.h_w_mat = np.identity(self.degree)
-            else:
-                self.h_w_mat = _check.pos_def_sym_mat(h_w_mat,'h_w_mat',ParameterFormatError)
-
-            if h_nu is None:
-                self.h_nu = float(self.degree)
-            else:
-                self.h_nu = _check.pos_float(h_nu,'h_nu',ParameterFormatError)
-
-        elif mu_vec is not None:
-            self.mu_vec = _check.float_vec(mu_vec,'mu_vec',ParameterFormatError)
-            self.degree = self.mu_vec.shape[0]
-            if lambda_mat is None:
-                self.lambda_mat = np.identity(self.degree)
-            else:
-                self.lambda_mat = _check.pos_def_sym_mat(lambda_mat,'lambda_mat',ParameterFormatError)
-
-            if h_m_vec is None:
-                self.h_m_vec = np.zeros(self.degree)
-            else:
-                self.h_m_vec = _check.float_vec(h_m_vec,'h_m_vec',ParameterFormatError)
-
-            if h_w_mat is None:
-                self.h_w_mat = np.identity(self.degree)
-            else:
-                self.h_w_mat = _check.pos_def_sym_mat(h_w_mat,'h_w_mat',ParameterFormatError)
-
-            if h_nu is None:
-                self.h_nu = float(self.degree)
-            else:
-                self.h_nu = _check.pos_float(h_nu,'h_nu',ParameterFormatError)
-
-        elif lambda_mat is not None:
-            self.lambda_mat = _check.pos_def_sym_mat(lambda_mat,'lambda_mat',ParameterFormatError)
-            self.degree = self.lambda_mat.shape[0]
-            self.mu_vec = np.zeros(self.degree)
-            if h_m_vec is None:
-                self.h_m_vec = np.zeros(self.degree)
-            else:
-                self.h_m_vec = _check.float_vec(h_m_vec,'h_m_vec',ParameterFormatError)
-
-            if h_w_mat is None:
-                self.h_w_mat = np.identity(self.degree)
-            else:
-                self.h_w_mat = _check.pos_def_sym_mat(h_w_mat,'h_w_mat',ParameterFormatError)
-        
-            if h_nu is None:
-                self.h_nu = float(self.degree)
-            else:
-                self.h_nu = _check.pos_float(h_nu,'h_nu',ParameterFormatError)
-
-        elif h_m_vec is not None:
-            self.h_m_vec = _check.float_vec(h_m_vec,'h_m_vec',ParameterFormatError)
-            self.degree = self.h_m_vec.shape[0]
-            self.mu_vec = np.zeros(self.degree)
-            self.lambda_mat = np.identity(self.degree)
-            if h_w_mat is None:
-                self.h_w_mat = np.identity(self.degree)
-            else:
-                self.h_w_mat = _check.pos_def_sym_mat(h_w_mat,'h_w_mat',ParameterFormatError)
-
-            if h_nu is None:
-                self.h_nu = float(self.degree)
-            else:
-                self.h_nu = _check.pos_float(h_nu,'h_nu',ParameterFormatError)
-
-        elif h_w_mat is not None:
-            self.h_w_mat = _check.pos_def_sym_mat(h_w_mat,'h_w_mat',ParameterFormatError)
-            self.degree = self.h_w_mat.shape[0]
-            self.mu_vec = np.zeros(self.degree)
-            self.lambda_mat = np.identity(self.degree)
-            self.h_m_vec = np.zeros(self.degree)
-
-            if h_nu is None:
-                self.h_nu = float(self.degree)
-            else:
-                self.h_nu = _check.pos_float(h_nu,'h_nu',ParameterFormatError)
-
-        elif h_nu is not None:
-            self.h_nu = _check.pos_float(h_nu,'h_nu',ParameterFormatError)
-            if self.h_nu <= 1.0:
-                self.degree = 1
-            else:
-                self.degree = 2
-            self.mu_vec = np.zeros(self.degree)
-            self.lambda_mat = np.identity(self.degree)
-            self.h_m_vec = np.zeros(self.degree)
-            self.h_w_mat = np.identity(self.degree)
-
-        else:
-            self.degree = 2
-            self.mu_vec = np.zeros(self.degree)
-            self.lambda_mat = np.identity(self.degree)
-            self.h_m_vec = np.zeros(self.degree)
-            self.h_w_mat = np.identity(self.degree)
-            self.h_nu = float(self.degree)
-
-        if (self.degree != self.mu_vec.shape[0]
-            or self.degree != self.lambda_mat.shape[0]
-            or self.degree != self.h_m_vec.shape[0]
-            or self.degree != self.h_w_mat.shape[0]
-            or self.h_nu <= self.degree - 1):
-                raise(ParameterFormatError(
-                    "degree and dimensions of mu_vec, lambda_mat,"
-                    +" h_m_vec, h_w_mat must be the same,"
-                    +" and degree must be smaller than h_nu + 1,"
-                    +" if two or more of them are specified."))
-
-        self.h_kappa = _check.pos_float(h_kappa,'h_kappa',ParameterFormatError)
+            self,
+            c_degree,
+            mu_vec=None,
+            lambda_mat=None,
+            h_m_vec=None,
+            h_kappa=1.0,
+            h_nu=None,
+            h_w_mat=None,
+            seed=None
+            ):
+        # constants
+        self.c_degree = _check.pos_int(c_degree,'c_degree',ParameterFormatError)
         self.rng = np.random.default_rng(seed)
+
+        # params
+        self.mu_vec = np.zeros(self.c_degree)
+        self.lambda_mat = np.eye(self.c_degree)
+
+        # h_params
+        self.h_m_vec = np.zeros(self.c_degree)
+        self.h_kappa = 1.0
+        self.h_nu = float(self.c_degree)
+        self.h_w_mat = np.eye(self.c_degree)
+
+        self.set_params(mu_vec,lambda_mat)
+        self.set_h_params(h_m_vec,h_kappa,h_nu,h_w_mat)
         
-    def set_h_params(self,h_m_vec,h_kappa,h_nu,h_w_mat):
+    def get_constants(self):
+        """Get constants of GenModel.
+
+        Returns
+        -------
+        constants : dict of {str: int}
+            * ``"c_degree"`` : the value of ``self.c_degree``
+        """
+        return {'c_degree':self.c_degree}
+
+    def set_h_params(self,h_m_vec=None,h_kappa=None,h_nu=None,h_w_mat=None):
         """Set the hyperparameters of the prior distribution.
 
         Parameters
         ----------
-        h_m_vec : numpy.ndarray
-            a vector of real numbers
-        h_kappa : float
-            a positive real number
-        h_nu : float
-            a real number > degree-1
-        h_w_mat : numpy.ndarray
-            a positive definite symetric matrix
+        h_m_vec : numpy.ndarray, optional
+            a vector of real numbers, by default None
+        h_kappa : float, optional
+            a positive real number, by default None
+        h_nu : float, optional
+            a real number > c_degree-1, by default None
+        h_w_mat : numpy.ndarray, optional
+            a positive definite symetric matrix, by default None
         """
-        self.h_m_vec = _check.float_vec(h_m_vec,'h_m_vec',ParameterFormatError)
-        self.h_kappa = _check.pos_float(h_kappa,'h_kappa',ParameterFormatError)
-        self.h_nu = _check.pos_float(h_nu,'h_nu',ParameterFormatError)
-        self.h_w_mat = _check.pos_def_sym_mat(h_w_mat,'h_w_mat',ParameterFormatError)
+        if h_m_vec is not None:
+            _check.float_vec(h_m_vec,'h_m_vec',ParameterFormatError)
+            _check.shape_consistency(
+                h_m_vec.shape[0],'h_m_vec.shape[0]',
+                self.c_degree,'self.c_degree',
+                ParameterFormatError
+            )
+            self.h_m_vec[:] = h_m_vec
 
-        if (self.h_m_vec.shape[0] != self.h_w_mat.shape[0]
-            or self.h_nu <= self.h_m_vec.shape[0] - 1):
+        if h_kappa is not None:
+            self.h_kappa = _check.pos_float(h_kappa,'h_kappa',ParameterFormatError)
+        
+        if h_nu is not None:
+            self.h_nu = _check.pos_float(h_nu,'h_nu',ParameterFormatError)
+            if h_nu <= self.c_degree - 1:
                 raise(ParameterFormatError(
-                    "They must be h_m_vec.shape[0] == h_w_mat.shape[0]"
-                    +" and h_nu > h_m_vec.shape[0] - 1."))
+                    "h_nu must be greater than self.c_degree - 1: "
+                    + f"self.c_degree = {self.c_degree}, h_nu = {h_nu}"))
+        
+        if h_w_mat is not None:
+            _check.pos_def_sym_mat(h_w_mat,'h_w_mat',ParameterFormatError)
+            _check.shape_consistency(
+                h_w_mat.shape[0],'h_w_mat.shape[0] and h_w_mat.shape[1]',
+                self.c_degree,'self.c_degree',
+                ParameterFormatError
+            )
+            self.h_w_mat[:] = h_w_mat
 
-        self.degree = self.h_m_vec.shape[0]
-        if self.degree != self.mu_vec.shape[0]:
-            self.mu_vec = np.zeros(self.degree)
-            warnings.warn("mu_vec is reinitialized to [0.0, 0.0, ..., 0.0] because dimension of mu_vec and h_params are mismatched.", ParameterFormatWarning)
-
-        if self.degree != self.lambda_mat.shape[0]:
-            self.lambda_mat = np.identity(self.degree)
-            warnings.warn("lambda_mat is reinitialized to the identity matrix because dimension of lambda_mat and h_params are mismatched.", ParameterFormatWarning)
+        return self
 
     def get_h_params(self):
         """Get the hyperparameters of the prior distribution.
@@ -239,29 +142,37 @@ class GenModel(base.Generative):
         """
         self.lambda_mat[:] = ss_wishart.rvs(df=self.h_nu,scale=self.h_w_mat,random_state=self.rng)
         self.mu_vec[:] = self.rng.multivariate_normal(mean=self.h_m_vec,cov=np.linalg.inv(self.h_kappa*self.lambda_mat))
-    
-    def set_params(self,mu_vec,lambda_mat):
+        return self
+
+    def set_params(self,mu_vec=None,lambda_mat=None):
         """Set the parameter of the sthocastic data generative model.
 
         Parameters
         ----------
-        mu_vec : numpy.ndarray
-            a vector of real numbers
-        lambda_mat : numpy.ndarray
-            a positive definite symetric matrix
+        mu_vec : numpy.ndarray, optional
+            a vector of real numbers, by default None
+        lambda_mat : numpy.ndarray, optional
+            a positive definite symetric matrix, by default None
         """
-        self.mu_vec = _check.float_vec(mu_vec,'mu_vec',ParameterFormatError)
-        self.lambda_mat = _check.pos_def_sym_mat(lambda_mat,'lambda_mat',ParameterFormatError)
-        if (self.mu_vec.shape[0] != self.lambda_mat.shape[0]):
-                raise(ParameterFormatError("They must be mu_vec.shape[0] == lambda_mat.shape[0]."))
+        if mu_vec is not None:
+            _check.float_vec(mu_vec,'mu_vec',ParameterFormatError)
+            _check.shape_consistency(
+                mu_vec.shape[0],'mu_vec.shape[0]',
+                self.c_degree,'self.c_degree',
+                ParameterFormatError
+            )
+            self.mu_vec[:] = mu_vec
+        
+        if lambda_mat is not None:
+            _check.pos_def_sym_mat(lambda_mat,'lambda_mat',ParameterFormatError)
+            _check.shape_consistency(
+                lambda_mat.shape[0],'lambda_mat.shape[0] and lambda_mat.shape[1]',
+                self.c_degree,'self.c_degree',
+                ParameterFormatError
+            )
+            self.lambda_mat[:] = lambda_mat
 
-        self.degree = self.mu_vec.shape[0]
-        if self.degree != self.h_m_vec.shape[0]:
-            self.h_m_vec = np.zeros(self.degree)
-            warnings.warn("h_m_vec is reinitialized to [0.0, 0.0, ..., 0.0] because dimension of h_m_vec and mu_vec are mismatched.", ParameterFormatWarning)
-        if self.degree != self.h_w_mat.shape[0]:
-            self.h_w_mat = np.identity(self.degree)
-            warnings.warn("h_w_mat is reinitialized to the identity matrix because dimension of h_w_mat and lambda_mat are mismatched.", ParameterFormatWarning)
+        return self
 
     def get_params(self):
         """Get the parameter of the sthocastic data generative model.
@@ -285,7 +196,7 @@ class GenModel(base.Generative):
         Returns
         -------
         x : numpy ndarray
-            2-dimensional array whose shape is ``(sammple_size,degree)`` and its elements are real number.
+            2-dimensional array whose shape is ``(sammple_size,c_degree)`` and its elements are real number.
         """
         _check.pos_int(sample_size,'sample_size',DataFormatError)
         return self.rng.multivariate_normal(mean=self.mu_vec,cov=np.linalg.inv(self.lambda_mat),size=sample_size)
@@ -320,7 +231,7 @@ class GenModel(base.Generative):
         Examples
         --------
         >>> from bayesml import multivariate_normal
-        >>> model = multivariate_normal.GenModel()
+        >>> model = multivariate_normal.GenModel(c_degree=2)
         >>> model.visualize_model()
         mu:
         [0. 0.]
@@ -330,7 +241,7 @@ class GenModel(base.Generative):
         
         .. image:: ./images/multivariate_normal_example.png
         """
-        if self.degree == 1:
+        if self.c_degree == 1:
             print(f"mu: {self.mu_vec}")
             print(f"lambda_mat: {self.lambda_mat}")
             lambda_mat_inv = np.linalg.inv(self.lambda_mat)
@@ -345,7 +256,7 @@ class GenModel(base.Generative):
             axes.set_ylabel("Density or frequency")
             plt.show()
 
-        elif self.degree == 2:
+        elif self.c_degree == 2:
             print(f"mu:\n{self.mu_vec}")
             print(f"lambda_mat:\n{self.lambda_mat}")
             lambda_mat_inv = np.linalg.inv(self.lambda_mat)
@@ -369,24 +280,21 @@ class GenModel(base.Generative):
             plt.show()
 
         else:
-            raise(ParameterFormatError("if degree > 2, it is impossible to visualize the model by this function."))
+            raise(ParameterFormatError("if c_degree > 2, it is impossible to visualize the model by this function."))
 
 class LearnModel(base.Posterior,base.PredictiveMixin):
     """The posterior distribution and the predictive distribution.
 
     Parameters
     ----------
-    degree : int, optional
-        a positive integer. Default is None, in which case 
-        a value consistent with ``h_m_vec``, ``h_w_mat``, 
-        and ``h_nu` is used. If all of them are not given, 
-        degree is assumed to be 2.
+    c_degree : int
+        a positive integer.
     h0_m_vec : numpy.ndarray, optional
         a vector of real numbers, by default [0.0, 0.0, ... , 0.0]
     h0_kappa : float, optional
         a positive real number, by default 1.0
     h0_nu : float, optional
-        a real number > degree-1, by default the value of ``degree``
+        a real number > c_degree-1, by default the value of ``c_degree``
     h0_w_mat : numpy.ndarray, optional
         a positive definite symetric matrix, by default the identity matrix
 
@@ -413,113 +321,103 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
     p_v_mat_inv : numpy.ndarray
         the inverse matrix of p_v_mat
     """
-    def __init__(self,degree=None,h0_m_vec=None,h0_kappa=1.0,h0_nu=None,h0_w_mat=None):
-        if degree is not None:
-            self.degree = _check.pos_int(degree,'degree',ParameterFormatError)
-            if h0_m_vec is None:
-                self.h0_m_vec = np.zeros(self.degree)
-            else:
-                self.h0_m_vec = _check.float_vec(h0_m_vec,'h0_m_vec',ParameterFormatError)
+    def __init__(
+            self,
+            c_degree,
+            h0_m_vec=None,
+            h0_kappa=1.0,
+            h0_nu=None,
+            h0_w_mat=None
+            ):
+        # constants
+        self.c_degree = _check.pos_int(c_degree,'c_degree',ParameterFormatError)
 
-            if h0_w_mat is None:
-                self.h0_w_mat = np.identity(self.degree)
-            else:
-                self.h0_w_mat = _check.pos_def_sym_mat(h0_w_mat,'h0_w_mat',ParameterFormatError)
-
-            if h0_nu is None:
-                self.h0_nu = float(self.degree)
-            else:
-                self.h0_nu = _check.pos_float(h0_nu,'h0_nu',ParameterFormatError)
+        # h0_params
+        self.h0_m_vec = np.zeros(self.c_degree)
+        self.h0_kappa = 1.0
+        self.h0_nu = float(self.c_degree)
+        self.h0_w_mat = np.eye(self.c_degree)
         
-        elif h0_m_vec is not None:
-            self.h0_m_vec = _check.float_vec(h0_m_vec,'h0_m_vec',ParameterFormatError)
-            self.degree = self.h0_m_vec.shape[0]
-            if h0_w_mat is None:
-                self.h0_w_mat = np.identity(self.degree)
-            else:
-                self.h0_w_mat = _check.pos_def_sym_mat(h0_w_mat,'h0_w_mat',ParameterFormatError)
+        self.h0_w_mat_inv = np.eye(self.c_degree)
 
-            if h0_nu is None:
-                self.h0_nu = float(self.degree)
-            else:
-                self.h0_nu = _check.pos_float(h0_nu,'h0_nu',ParameterFormatError)
+        # hn_params
+        self.hn_m_vec = np.zeros(self.c_degree)
+        self.hn_kappa = 1.0
+        self.hn_nu = float(self.c_degree)
+        self.hn_w_mat = np.eye(self.c_degree)
         
-        elif h0_w_mat is not None:
-            self.h0_w_mat = _check.pos_def_sym_mat(h0_w_mat,'h0_w_mat',ParameterFormatError)
-            self.degree = self.h0_w_mat.shape[0]
-            self.h0_m_vec = np.zeros(self.degree)
-            if h0_nu is None:
-                self.h0_nu = float(self.degree)
-            else:
-                self.h0_nu = _check.pos_float(h0_nu,'h0_nu',ParameterFormatError)
-        
-        elif h0_nu is not None:
-            self.h0_nu = _check.pos_float(h0_nu,'h0_nu',ParameterFormatError)
-            if self.h0_nu <= 1.0:
-                self.degree = 1
-            else:
-                self.degree = 2
-            self.h0_m_vec = np.zeros(self.degree)
-            self.h0_w_mat = np.identity(self.degree)
-        
-        else:
-            self.degree = 2
-            self.h0_m_vec = np.zeros(self.degree)
-            self.h0_w_mat = np.identity(self.degree)
-            self.h0_nu = float(self.degree)
+        self.hn_w_mat_inv = np.eye(self.c_degree)
 
-        if (self.degree != self.h0_m_vec.shape[0]
-            or self.degree != self.h0_w_mat.shape[0]
-            or self.h0_nu <= self.degree - 1):
-                raise(ParameterFormatError(
-                    "degree and dimensions of"
-                    +" h0_m_vec, h0_w_mat must be the same,"
-                    +" and degree must be smaller than h0_nu + 1,"
-                    +" if two or more of them are specified."))
+        # p_params
+        self.p_m_vec = np.zeros(self.c_degree)
+        self.p_nu = 1.0
+        self.p_v_mat = np.eye(self.c_degree)/2.0
 
-        self.h0_kappa = _check.pos_float(h0_kappa,'h0_kappa',ParameterFormatError)
-        self.h0_w_mat_inv = np.linalg.inv(self.h0_w_mat)
+        self.p_v_mat_inv = np.eye(self.c_degree)*2.0
 
-        self.hn_m_vec = np.copy(self.h0_m_vec)
-        self.hn_kappa = self.h0_kappa
-        self.hn_nu = self.h0_nu
-        self.hn_w_mat = np.copy(self.h0_w_mat)
-        self.hn_w_mat_inv = np.copy(self.h0_w_mat_inv)
+        self.set_h0_params(
+            h0_m_vec,
+            h0_kappa,
+            h0_nu,
+            h0_w_mat,
+        )
 
-        self.p_m_vec = np.copy(self.hn_m_vec)
-        self.p_nu = self.hn_nu - self.degree + 1
-        self.p_v_mat = self.hn_kappa*self.p_nu/(self.hn_kappa+1) * self.hn_w_mat
-        self.p_v_mat_inv = (self.hn_kappa+1)/self.hn_kappa/self.p_nu * self.hn_w_mat_inv
+    def get_constants(self):
+        """Get constants of LearnModel.
 
-    def set_h0_params(self,h0_m_vec,h0_kappa,h0_nu,h0_w_mat):
+        Returns
+        -------
+        constants : dict of {str: int}
+            * ``"c_degree"`` : the value of ``self.c_degree``
+        """
+        return {'c_degree':self.c_degree}
+
+    def set_h0_params(self,h0_m_vec=None,h0_kappa=None,h0_nu=None,h0_w_mat=None):
         """Set the hyperparameters of the prior distribution.
         
         Parameters
         ----------
-        h0_m_vec : numpy.ndarray
-            a vector of real numbers
-        h0_kappa : float
-            a positive real number
-        h0_nu : float
-            a real number > degree-1
-        h0_w_mat : numpy.ndarray
-            a positive definite symetric matrix
+        h0_m_vec : numpy.ndarray, optional
+            a vector of real numbers, by default None
+        h0_kappa : float, optional
+            a positive real number, by default None
+        h0_nu : float, optional
+            a real number > c_degree-1, by default None
+        h0_w_mat : numpy.ndarray, optional
+            a positive definite symetric matrix, by default None
         """
-        self.h0_m_vec = _check.float_vec(h0_m_vec,'h0_m_vec',ParameterFormatError)
-        self.h0_kappa = _check.pos_float(h0_kappa,'h0_kappa',ParameterFormatError)
-        self.h0_nu = _check.pos_float(h0_nu,'h0_nu',ParameterFormatError)
-        self.h0_w_mat = _check.pos_def_sym_mat(h0_w_mat,'h0_w_mat',ParameterFormatError)
+        if h0_m_vec is not None:
+            _check.float_vec(h0_m_vec,'h0_m_vec',ParameterFormatError)
+            _check.shape_consistency(
+                h0_m_vec.shape[0],'h0_m_vec.shape[0]',
+                self.c_degree,'self.c_degree',
+                ParameterFormatError
+            )
+            self.h0_m_vec[:] = h0_m_vec
 
-        if (self.h0_m_vec.shape[0] != self.h0_w_mat.shape[0]
-            or self.h0_nu <= self.h0_m_vec.shape[0] - 1):
+        if h0_kappa is not None:
+            self.h0_kappa = _check.pos_float(h0_kappa,'h0_kappa',ParameterFormatError)
+        
+        if h0_nu is not None:
+            self.h0_nu = _check.pos_float(h0_nu,'h0_nu',ParameterFormatError)
+            if h0_nu <= self.c_degree - 1:
                 raise(ParameterFormatError(
-                    "They must be h0_m_vec.shape[0] == h0_w_mat.shape[0]"
-                    +" and h0_nu > h0_m_vec.shape[0] - 1."))
+                    "h0_nu must be greater than self.c_degree - 1: "
+                    + f"self.c_degree = {self.c_degree}, h0_nu = {h0_nu}"))
+        
+        if h0_w_mat is not None:
+            _check.pos_def_sym_mat(h0_w_mat,'h0_w_mat',ParameterFormatError)
+            _check.shape_consistency(
+                h0_w_mat.shape[0],'h0_w_mat.shape[0] and h0_w_mat.shape[1]',
+                self.c_degree,'self.c_degree',
+                ParameterFormatError
+            )
+            self.h0_w_mat[:] = h0_w_mat
 
         self.h0_w_mat_inv = np.linalg.inv(self.h0_w_mat)
-        self.degree = self.h0_m_vec.shape[0]
 
         self.reset_hn_params()
+        return self
 
     def get_h0_params(self):
         """Get the initial values of the hyperparameters of the posterior distribution.
@@ -534,35 +432,52 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         """
         return {"h0_m_vec":self.h0_m_vec, "h0_kappa":self.h0_kappa, "h0_nu":self.h0_nu, "h0_w_mat":self.h0_w_mat}
     
-    def set_hn_params(self,hn_m_vec,hn_kappa,hn_nu,hn_w_mat):
+    def set_hn_params(self,hn_m_vec=None,hn_kappa=None,hn_nu=None,hn_w_mat=None):
         """Set updated values of the hyperparameter of the posterior distribution.
 
         Parameters
         ----------
-        hn_m_vec : numpy.ndarray
-            a vector of real numbers
-        hn_kappa : float
-            a positive real number
-        hn_nu : float
-            a real number > degree-1
-        hn_w_mat : numpy.ndarray
-            a positive definite symetric matrix
+        hn_m_vec : numpy.ndarray, optional
+            a vector of real numbers, by default None
+        hn_kappa : float, optional
+            a positive real number, by default None
+        hn_nu : float, optional
+            a real number > c_degree-1, by default None
+        hn_w_mat : numpy.ndarray, optional
+            a positive definite symetric matrix, by default None
         """
-        self.hn_m_vec = _check.float_vec(hn_m_vec,'hn_m_vec',ParameterFormatError)
-        self.hn_kappa = _check.pos_float(hn_kappa,'hn_kappa',ParameterFormatError)
-        self.hn_nu = _check.pos_float(hn_nu,'hn_nu',ParameterFormatError)
-        self.hn_w_mat = _check.pos_def_sym_mat(hn_w_mat,'hn_w_mat',ParameterFormatError)
+        if hn_m_vec is not None:
+            _check.float_vec(hn_m_vec,'hn_m_vec',ParameterFormatError)
+            _check.shape_consistency(
+                hn_m_vec.shape[0],'hn_m_vec.shape[0]',
+                self.c_degree,'self.c_degree',
+                ParameterFormatError
+            )
+            self.hn_m_vec[:] = hn_m_vec
 
-        if (self.hn_m_vec.shape[0] != self.hn_w_mat.shape[0]
-            or self.hn_nu <= self.hn_m_vec.shape[0] - 1):
+        if hn_kappa is not None:
+            self.hn_kappa = _check.pos_float(hn_kappa,'hn_kappa',ParameterFormatError)
+        
+        if hn_nu is not None:
+            self.hn_nu = _check.pos_float(hn_nu,'hn_nu',ParameterFormatError)
+            if hn_nu <= self.c_degree - 1:
                 raise(ParameterFormatError(
-                    "They must be hn_m_vec.shape[0] == hn_w_mat.shape[0]"
-                    +" and hn_nu > hn_m_vec.shape[0] - 1."))
+                    "hn_nu must be greater than self.c_degree - 1: "
+                    + f"self.c_degree = {self.c_degree}, hn_nu = {hn_nu}"))
+        
+        if hn_w_mat is not None:
+            _check.pos_def_sym_mat(hn_w_mat,'hn_w_mat',ParameterFormatError)
+            _check.shape_consistency(
+                hn_w_mat.shape[0],'hn_w_mat.shape[0] and hn_w_mat.shape[1]',
+                self.c_degree,'self.c_degree',
+                ParameterFormatError
+            )
+            self.hn_w_mat[:] = hn_w_mat
 
         self.hn_w_mat_inv = np.linalg.inv(self.hn_w_mat)
-        self.degree = self.hn_m_vec.shape[0]
 
         self.calc_pred_dist()
+        return self
 
     def get_hn_params(self):
         """Get the hyperparameters of the posterior distribution.
@@ -577,33 +492,11 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         """
         return {"hn_m_vec":self.hn_m_vec, "hn_kappa":self.hn_kappa, "hn_nu":self.hn_nu, "hn_w_mat":self.hn_w_mat}
     
-    def reset_hn_params(self):
-        """Reset the hyperparameters of the posterior distribution to their initial values.
-        
-        They are reset to `self.h0_m_vec`, `self.h0_kappa`, `self.h0_nu` and `self.h0_w_hat`.
-        Note that the parameters of the predictive distribution are also calculated from `self.h0_m_vec`, `self.h0_kappa`, `self.h0_nu` and `self.h0_w_hat`.
-        """
-        self.hn_m_vec = np.copy(self.h0_m_vec)
-        self.hn_kappa = self.h0_kappa
-        self.hn_nu = self.h0_nu
-        self.hn_w_mat = np.copy(self.h0_w_mat)
-        self.hn_w_mat_inv = np.copy(self.h0_w_mat_inv)
-
-        self.calc_pred_dist()
-    
-    def overwrite_h0_params(self):
-        """Overwrite the initial values of the hyperparameters of the posterior distribution by the learned values.
-        
-        They are overwitten by `self.hn_m_vec`, `self.hn_kappa`, `self.hn_nu` and `self.hn_w_mat`.
-        Note that the parameters of the predictive distribution are also calculated from `self.hn_m_vec`, `self.hn_kappa`, `self.hn_nu` and `self.hn_w_mat`.
-        """
-        self.h0_m_vec = np.copy(self.hn_m_vec)
-        self.h0_kappa = self.hn_kappa
-        self.h0_nu = self.hn_nu
-        self.h0_w_mat = np.copy(self.hn_w_mat)
-        self.h0_w_mat_inv = np.copy(self.hn_w_mat_inv)
-
-        self.calc_pred_dist()
+    def _check_sample(self,x):
+        _check.float_vecs(x,'x',DataFormatError)
+        if x.shape[-1] != self.c_degree:
+            raise(DataFormatError(f"x.shape[-1] must be c_degree:{self.c_degree}"))
+        return x.reshape(-1,self.c_degree)
 
     def update_posterior(self,x):
         """Update the hyperparameters of the posterior distribution using traning data.
@@ -613,10 +506,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         x : numpy.ndarray
             All the elements must be real number.
         """
-        _check.float_vecs(x,'x',DataFormatError)
-        if x.shape[-1] != self.degree:
-            raise(DataFormatError(f"x.shape[-1] must be degree:{self.degree}"))
-        x = x.reshape(-1,self.degree)
+        x = self._check_sample(x)
 
         n = x.shape[0]
         x_bar = x.sum(axis=0)/n
@@ -629,6 +519,22 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         self.hn_nu += n
 
         self.hn_w_mat[:] = np.linalg.inv(self.hn_w_mat_inv) 
+        return self
+
+    def _update_posterior(self,x):
+        """Update opsterior without input check."""
+        n = x.shape[0]
+        x_bar = x.sum(axis=0)/n
+
+        self.hn_w_mat_inv[:] = (self.hn_w_mat_inv + (x-x_bar).T @ (x-x_bar)
+                                + (x_bar - self.hn_m_vec)[:,np.newaxis] @ (x_bar - self.hn_m_vec)[np.newaxis,:]
+                                  * self.hn_kappa * n / (self.hn_kappa + n))
+        self.hn_m_vec[:] = (self.hn_kappa*self.hn_m_vec + n*x_bar) / (self.hn_kappa+n)
+        self.hn_kappa += n
+        self.hn_nu += n
+
+        self.hn_w_mat[:] = np.linalg.inv(self.hn_w_mat_inv) 
+        return self
 
     def estimate_params(self,loss="squared",dict_out=False):
         """Estimate the parameter of the stochastic data generative model under the given criterion.
@@ -665,11 +571,11 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
             else:
                 return self.hn_m_vec, self.hn_nu * self.hn_w_mat
         elif loss == "0-1":
-            if self.hn_nu >= self.degree + 1:
+            if self.hn_nu >= self.c_degree + 1:
                 if dict_out:
-                    return {'mu_vec':self.hn_m_vec,'lambda_mat':(self.hn_nu - self.degree - 1) * self.hn_w_mat}
+                    return {'mu_vec':self.hn_m_vec,'lambda_mat':(self.hn_nu - self.c_degree - 1) * self.hn_w_mat}
                 else:
-                    return self.hn_m_vec, (self.hn_nu - self.degree - 1) * self.hn_w_mat
+                    return self.hn_m_vec, (self.hn_nu - self.c_degree - 1) * self.hn_w_mat
             else:
                 warnings.warn("MAP estimate of lambda_mat doesn't exist for the current hn_nu.",ResultWarning)
                 if dict_out:
@@ -678,8 +584,8 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
                     return self.hn_m_vec, None
         elif loss == "KL":
             return (ss_multivariate_t(loc=self.hn_m_vec,
-                                        shape=self.hn_w_mat_inv / self.hn_kappa / (self.hn_nu - self.degree + 1),
-                                        df=self.hn_nu - self.degree + 1),
+                                        shape=self.hn_w_mat_inv / self.hn_kappa / (self.hn_nu - self.c_degree + 1),
+                                        df=self.hn_nu - self.c_degree + 1),
                     ss_wishart(df=self.hn_nu,scale=self.hn_w_mat))
         else:
             raise(CriteriaError("Unsupported loss function! "
@@ -691,9 +597,9 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         Examples
         --------
         >>> from bayesml import multivariate_normal
-        >>> gen_model = multivariate_normal.GenModel()
+        >>> gen_model = multivariate_normal.GenModel(c_degree=2)
         >>> x = gen_model.gen_sample(100)
-        >>> learn_model = multivariate_normal.LearnModel()
+        >>> learn_model = multivariate_normal.LearnModel(c_degree=2)
         >>> learn_model.update_posterior(x)
         >>> learn_model.visualize_posterior()
         hn_m_vec:
@@ -722,7 +628,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         print("E[lambda_mat]=")
         print(f"{self.hn_nu * self.hn_w_mat}")
         mu_vec_pdf, lambda_mat_pdf = self.estimate_params(loss="KL")
-        if self.degree == 1:
+        if self.c_degree == 1:
             fig, axes = plt.subplots(1,2)
             # for mu_vec
             x = np.linspace(self.hn_m_vec[0]-4.0*np.sqrt((self.hn_w_mat_inv / self.hn_kappa / self.hn_nu)[0,0]),
@@ -743,7 +649,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
             fig.tight_layout()
             plt.show()
 
-        elif self.degree == 2:
+        elif self.c_degree == 2:
             fig, axes = plt.subplots()
             x = np.linspace(self.hn_m_vec[0]-3.0*np.sqrt((self.hn_w_mat_inv / self.hn_kappa / self.hn_nu)[0,0]),
                             self.hn_m_vec[0]+3.0*np.sqrt((self.hn_w_mat_inv / self.hn_kappa / self.hn_nu)[0,0]),
@@ -762,7 +668,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
             plt.show()
 
         else:
-            raise(ParameterFormatError("if degree > 2, it is impossible to visualize the model by this function."))
+            raise(ParameterFormatError("if c_degree > 2, it is impossible to visualize the model by this function."))
     
     def get_p_params(self):
         """Get the parameters of the predictive distribution.
@@ -778,10 +684,17 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
     
     def calc_pred_dist(self):
         """Calculate the parameters of the predictive distribution."""
-        self.p_m_vec = np.copy(self.hn_m_vec)
-        self.p_nu = self.hn_nu - self.degree + 1
-        self.p_v_mat = self.hn_kappa*self.p_nu/(self.hn_kappa+1) * self.hn_w_mat
-        self.p_v_mat_inv = (self.hn_kappa+1)/self.hn_kappa/self.p_nu * self.hn_w_mat_inv
+        self.p_m_vec[:] = self.hn_m_vec
+        self.p_nu = self.hn_nu - self.c_degree + 1
+        self.p_v_mat[:] = self.hn_kappa*self.p_nu/(self.hn_kappa+1) * self.hn_w_mat
+        self.p_v_mat_inv[:] = (self.hn_kappa+1)/self.hn_kappa/self.p_nu * self.hn_w_mat_inv
+        return self
+    
+    def _calc_pred_density(self,x):
+        return ss_multivariate_t.pdf(x,
+                                     loc=self.p_m_vec,
+                                     shape=self.p_v_mat_inv,
+                                     df=self.p_nu)
 
     def make_prediction(self,loss="squared"):
         """Predict a new data point under the given criterion.
@@ -820,7 +733,7 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
         Parameters
         ----------
         x : numpy.ndarray
-            It must be a degree-dimensional vector
+            It must be a c_degree-dimensional vector
         loss : str, optional
             Loss function underlying the Bayes risk function, by default \"squared\".
             This function supports \"squared\", \"0-1\", and \"KL\".
@@ -831,8 +744,8 @@ class LearnModel(base.Posterior,base.PredictiveMixin):
             The predicted value under the given loss function. 
         """
         _check.float_vec(x,'x',DataFormatError)
-        if x.shape != (self.degree,):
-            raise(DataFormatError(f"x must be a 1-dimensional float array whose size is degree: {self.degree}."))
+        if x.shape != (self.c_degree,):
+            raise(DataFormatError(f"x must be a 1-dimensional float array whose size is c_degree: {self.c_degree}."))
         self.calc_pred_dist()
         prediction = self.make_prediction(loss=loss)
         self.update_posterior(x[np.newaxis,:])
